@@ -79,6 +79,7 @@ typedef struct _cairo_xcb_xid {
 #define XCB_RENDER_HAS_FILTERS(surface)			XCB_RENDER_AT_LEAST((surface), 0, 6)
 #define XCB_RENDER_HAS_FILTER_GOOD(surface) FALSE
 #define XCB_RENDER_HAS_FILTER_BEST(surface) FALSE
+#define XCB_RENDER_HAS_SUBPIXEL_ORDER(surface)		XCB_RENDER_AT_LEAST((surface), 0, 6)
 
 #define XCB_RENDER_HAS_EXTENDED_REPEAT(surface)	XCB_RENDER_AT_LEAST((surface), 0, 10)
 #define XCB_RENDER_HAS_GRADIENTS(surface)	XCB_RENDER_AT_LEAST((surface), 0, 10)
@@ -110,7 +111,7 @@ _cairo_xcb_connection_find_visual_formats (cairo_xcb_connection_t *connection,
 		cairo_xcb_xrender_format_t *f;
 		cairo_status_t status;
 
-		f = malloc (sizeof (cairo_xcb_xrender_format_t));
+		f = _cairo_malloc (sizeof (cairo_xcb_xrender_format_t));
 		if (unlikely (f == NULL))
 		    return _cairo_error (CAIRO_STATUS_NO_MEMORY);
 
@@ -176,7 +177,7 @@ _cairo_xcb_connection_parse_xrender_formats (cairo_xcb_connection_t *connection,
 	    if (! _cairo_hash_table_lookup (connection->xrender_formats, &key)) {
 		cairo_xcb_xrender_format_t *f;
 
-		f = malloc (sizeof (cairo_xcb_xrender_format_t));
+		f = _cairo_malloc (sizeof (cairo_xcb_xrender_format_t));
 		if (unlikely (f == NULL))
 		    return _cairo_error (CAIRO_STATUS_NO_MEMORY);
 
@@ -254,7 +255,7 @@ pixmap_depths_usable (cairo_xcb_connection_t *connection,
     cairo_bool_t success = TRUE;
     int depth, i, j;
 
-    pixmap = _cairo_xcb_connection_get_xid (connection);
+    pixmap = xcb_generate_id (connection->xcb_connection);
 
     for (depth = 1, i = 0; depth <= 32; depth++) {
 	if (missing & DEPTH_MASK(depth)) {
@@ -274,8 +275,6 @@ pixmap_depths_usable (cairo_xcb_connection_t *connection,
 	free (create_error);
     }
 
-    _cairo_xcb_connection_put_xid (connection, pixmap);
-
     return success;
 }
 
@@ -407,6 +406,15 @@ _cairo_xcb_connection_query_render (cairo_xcb_connection_t *connection)
     if (XCB_RENDER_HAS_GRADIENTS (version))
 	connection->flags |= CAIRO_XCB_RENDER_HAS_GRADIENTS;
 
+    if (XCB_RENDER_HAS_SUBPIXEL_ORDER (version)) {
+	uint32_t screen;
+	uint32_t *subpixel = xcb_render_query_pict_formats_subpixels(formats);
+
+	/* The spec explicitly allows to have too few entries in the reply... */
+	for (screen = 0; screen < formats->num_subpixel && screen < connection->root->roots_len; screen++)
+	    connection->subpixel_orders[screen] = subpixel[screen];
+    }
+
     free (version);
 
     status = _cairo_xcb_connection_parse_xrender_formats (connection, formats);
@@ -452,10 +460,9 @@ can_use_shm (cairo_xcb_connection_t *connection)
 	return FALSE;
     }
 
-    shmseg = _cairo_xcb_connection_get_xid (connection);
+    shmseg = xcb_generate_id (connection->xcb_connection);
     cookie[0] = xcb_shm_attach_checked (c, shmseg, shmid, FALSE);
     cookie[1] = xcb_shm_detach_checked (c, shmseg);
-    _cairo_xcb_connection_put_xid (connection, shmseg);
 
     error = xcb_request_check (c, cookie[0]);
     if (error != NULL)
@@ -576,11 +583,10 @@ _device_destroy (void *device)
 #endif
     _cairo_freepool_fini (&connection->shm_info_freelist);
 
-    _cairo_freepool_fini (&connection->xid_pool);
-
     CAIRO_MUTEX_FINI (connection->shm_mutex);
     CAIRO_MUTEX_FINI (connection->screens_mutex);
 
+    free (connection->subpixel_orders);
     free (connection);
 }
 
@@ -623,7 +629,7 @@ _cairo_xcb_connection_get (xcb_connection_t *xcb_connection)
 	}
     }
 
-    connection = malloc (sizeof (cairo_xcb_connection_t));
+    connection = _cairo_malloc (sizeof (cairo_xcb_connection_t));
     if (unlikely (connection == NULL))
 	goto unlock;
 
@@ -651,10 +657,6 @@ _cairo_xcb_connection_get (xcb_connection_t *xcb_connection)
 	goto unlock;
     }
 
-    cairo_list_init (&connection->free_xids);
-    _cairo_freepool_init (&connection->xid_pool,
-			  sizeof (cairo_xcb_xid_t));
-
     cairo_list_init (&connection->shm_pools);
     cairo_list_init (&connection->shm_pending);
     _cairo_freepool_init (&connection->shm_info_freelist,
@@ -684,6 +686,14 @@ _cairo_xcb_connection_get (xcb_connection_t *xcb_connection)
 
     connection->root = xcb_get_setup (xcb_connection);
     connection->render = NULL;
+    connection->subpixel_orders = calloc (connection->root->roots_len, sizeof(*connection->subpixel_orders));
+    if (unlikely (connection->subpixel_orders == NULL)) {
+	CAIRO_MUTEX_UNLOCK (connection->device.mutex);
+	_cairo_xcb_connection_destroy (connection);
+	connection = NULL;
+	goto unlock;
+    }
+
     ext = xcb_get_extension_data (xcb_connection, &xcb_render_id);
     if (ext != NULL && ext->present) {
 	status = _cairo_xcb_connection_query_render (connection);
@@ -747,43 +757,6 @@ _cairo_xcb_connection_get_xrender_format_for_visual (cairo_xcb_connection_t *con
     return format ? format->xrender_format : XCB_NONE;
 }
 
-void
-_cairo_xcb_connection_put_xid (cairo_xcb_connection_t *connection,
-			       uint32_t xid)
-{
-    cairo_xcb_xid_t *cache;
-
-    assert (CAIRO_MUTEX_IS_LOCKED (connection->device.mutex));
-    cache = _cairo_freepool_alloc (&connection->xid_pool);
-    if (likely (cache != NULL)) {
-	cache->xid = xid;
-	cairo_list_add (&cache->link, &connection->free_xids);
-    }
-}
-
-uint32_t
-_cairo_xcb_connection_get_xid (cairo_xcb_connection_t *connection)
-{
-    uint32_t xid;
-
-    assert (CAIRO_MUTEX_IS_LOCKED (connection->device.mutex));
-    if (! cairo_list_is_empty (&connection->free_xids)) {
-	cairo_xcb_xid_t *cache;
-
-	cache = cairo_list_first_entry (&connection->free_xids,
-					cairo_xcb_xid_t,
-					link);
-	xid = cache->xid;
-
-	cairo_list_del (&cache->link);
-	_cairo_freepool_free (&connection->xid_pool, cache);
-    } else {
-	xid = xcb_generate_id (connection->xcb_connection);
-    }
-
-    return xid;
-}
-
 /**
  * cairo_xcb_device_get_connection:
  * @device: a #cairo_device_t for the XCB backend
