@@ -33,15 +33,6 @@
  * Contributor(s):
  */
 
-#define WIN32_LEAN_AND_MEAN
-/* We require Windows 2000 features such as GetGlyphIndices */
-#if !defined(WINVER) || (WINVER < 0x0500)
-# define WINVER 0x0500
-#endif
-#if !defined(_WIN32_WINNT) || (_WIN32_WINNT < 0x0500)
-# define _WIN32_WINNT 0x0500
-#endif
-
 #include "cairoint.h"
 
 #include "cairo-win32-private.h"
@@ -80,7 +71,6 @@
  *
  * Note: Win32 GDI fonts do not support color fonts. Use DWrite fonts
  * if color font support is required.
-
  **/
 
 /**
@@ -1115,6 +1105,12 @@ _cairo_win32_scaled_font_glyph_init (void		       *abstract_font,
 	    return status;
     }
 
+    if (info & CAIRO_SCALED_GLYPH_INFO_COLOR_SURFACE) {
+	scaled_glyph->color_glyph = FALSE;
+	scaled_glyph->color_glyph_set = TRUE;
+	return CAIRO_INT_STATUS_UNSUPPORTED;
+    }
+
     if (info & CAIRO_SCALED_GLYPH_INFO_SURFACE) {
 	status = _cairo_win32_scaled_font_init_glyph_surface (scaled_font, scaled_glyph);
 	if (status)
